library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

--
-- xr16 Example Project
-- Doug Hodson, Jun 2003
-- www.RetroMicro.com
--
-- This project must be run at 25 MHz
--
entity chipIO is
	
	port(
		pin_sysclk       : in std_logic;   -- system clock
		pin_flash_ce_n   : out std_logic;  -- Flash RAM chip-enable
      pin_led          : out std_logic_vector(6 downto 0);  -- 7-seg LED

		-- SDRAM pin connections
		pin_sclkfb : in std_logic;                   -- feedback SDRAM clock with PCB delays
		pin_sclk   : out std_logic;                  -- clock to SDRAM
		pin_cke    : out std_logic;                  -- SDRAM clock-enable
		pin_cs_n   : out std_logic;                  -- SDRAM chip-select
		pin_ras_n  : out std_logic;                  -- SDRAM RAS
		pin_cas_n  : out std_logic;                  -- SDRAM CAS
		pin_we_n   : out std_logic;                  -- SDRAM write-enable
		pin_ba     : out unsigned( 1 downto 0);      -- SDRAM bank-address
		pin_sAddr  : out unsigned(11 downto 0);      -- SDRAM address bus
		pin_sData  : inout unsigned(16-1 downto 0);  -- data bus to SDRAM
		pin_dqmh   : out std_logic;                  -- SDRAM DQMH
		pin_dqml   : out std_logic                   -- SDRAM DQML			
	);
	
end chipIO;

architecture arch of chipIO is

	constant YES:	std_logic := '1';
	constant NO:	std_logic := '0';
	constant HI:	std_logic := '1';
	constant LO:	std_logic := '0';

	constant DATA_WIDTH: natural := 16;
	constant ADDR_WIDTH: natural := 23;

   signal sysClk   : std_logic;  -- system clock
   signal sysReset : std_logic;  -- system reset

	-- xr16 component
	component xr16
		port(
		clk      : in std_logic;       -- global clock
		rst      : in std_logic;       -- global reset
		rdy      : in std_logic;       -- current memory access if ready
		ud_t     : in std_logic;       -- active low store data MSB output enable
		ld_t     : in std_logic;       -- active low store data LSB output enable
		udlt_t   : in std_logic;       -- active low store data MSB->LSB output en
		int_req  : in std_logic;       -- interrupt request
		dma_req  : in std_logic;       -- DMA request
		zerodma  : in std_logic;       -- zero DMA counter request
		insn     : in std_logic_vector(15 downto 0);     -- new instruction word
		mem_ce   : out std_logic;      -- memory access clock enable
		word_nxt : out std_logic;      -- next access is word wide
		read_nxt : out std_logic;      -- next access is read
		dbus_nxt : out std_logic;      -- next access uses on-chip data bus
		dma      : out std_logic;      -- current access is a DMA transfer
		addr_nxt : out std_logic_vector(15 downto 0);    -- address of next memory access
		d        : inout std_logic_vector(15 downto 0)   --on-chip data bus
		);
	end component;

	-- xr16 signals
	signal xr16_rdy      : std_logic;
	signal xr16_ud_t     : std_logic;
	signal xr16_ld_t     : std_logic;
	signal xr16_insn     : std_logic_vector(15 downto 0);
	signal xr16_mem_ce   : std_logic;
	signal xr16_read_nxt : std_logic;
	signal xr16_dbus_nxt : std_logic;
	signal xr16_addr_nxt : std_logic_vector(15 downto 0);
	signal xr16_d        : std_logic_vector(15 downto 0);
	
	component sdramCntl
		generic(
			FREQ:	natural := 25_000;		-- operating frequency in KHz
			DATA_WIDTH: natural := 16;		-- host & SDRAM data width
			NROWS: natural := 4096;			-- number of rows in SDRAM array
			NCOLS: natural := 512;			-- number of columns in SDRAM array
			HADDR_WIDTH: natural := 23;	-- host-side address width
			SADDR_WIDTH: natural := 12		-- SDRAM-side address width
		);
		port(
			clkin:	in	std_logic;				-- master clock

			-- host side
			bufclk:	out std_logic;				-- buffered master clock
			clk0:		out std_logic;				-- host clock sync'ed to master clock
			clk2x:	out std_logic;				-- double-speed host clock
			lock:		out std_logic;				-- indicate when clock circuitry is locked to master clock
			rst:		in	std_logic;				-- reset
			rd:			in	std_logic;				-- read data
			wr:			in	std_logic;				-- write data
			done:		out	std_logic;				-- read/write op done
			hAddr:	in	unsigned(HADDR_WIDTH-1 downto 0);					-- address from host
			hDIn:		in	unsigned(DATA_WIDTH-1 downto 0);					-- data from host
			hDOut:	out	unsigned(DATA_WIDTH-1 downto 0);					-- data to host
			sdramCntl_state: out std_logic_vector(3 downto 0);		

			-- SDRAM side
			sclkfb:	in	std_logic;				-- clock from SDRAM after PCB delays
			sclk:		out std_logic;				-- SDRAM clock sync'ed to master clock
			cke:		out	std_logic;				-- clock-enable to SDRAM
			cs_n:		out	std_logic;				-- chip-select to SDRAM
			ras_n:	out	std_logic;				-- command input to SDRAM
			cas_n:	out	std_logic;				-- command input to SDRAM
			we_n:		out	std_logic;				-- command input to SDRAM
			ba:			out	unsigned(1 downto 0);	-- SDRAM bank address bits
			sAddr:	out	unsigned(SADDR_WIDTH-1 downto 0);					-- SDRAM row/column address
			sData:	inout unsigned(DATA_WIDTH-1 downto 0);				-- SDRAM in/out databus
			dqmh:		out	std_logic;				-- high databits I/O mask
			dqml:		out	std_logic					-- low databits I/O mask
		);
	end component;

	
	-- SDRAM host side signals
	signal sdram_bufclk : std_logic;    -- buffered master clock output
	signal sdram_lock   : std_logic;    -- SDRAM clock DLL lock indicator
	signal sdram_rd     : std_logic;    -- host-side read control signal
	signal sdram_wr     : std_logic;    -- host-side write control signal
	signal sdram_done   : std_logic;    -- SDRAM operation complete indicator
	signal sdram_hAddr  : unsigned(ADDR_WIDTH-1 downto 0);  -- host address bus
	signal sdram_hDIn   : unsigned(DATA_WIDTH-1 downto 0);	-- host-side data to SDRAM
	signal sdram_hDOut  : unsigned(DATA_WIDTH-1 downto 0);	-- host-side data from SDRAM

   signal ledDigit : natural;
	
	-- memory state management variables
	type memoryStateType is (
		MEM_ACTIVE,
		MEM_READY
	);
	signal memoryState : memoryStateType;

	signal addr_r  : unsigned(ADDR_WIDTH-1 downto 0);  -- current address
	signal ramrd, ramwr : std_logic;
	signal read_r : std_logic;
	signal readRAMData, writeRAMData : std_logic;
		
begin

	pin_flash_ce_n <= '1';    -- disable Flash RAM

	xr16_d <= std_logic_vector(sdram_hDOut) when (readRAMData=YES) else (others=>'Z');
	sdram_hDIn <= unsigned(xr16_d) when (writeRAMData=YES) else (others=>'Z');

	sdram_hAddr <= addr_r;
	xr16_insn <= std_logic_vector(sdram_hDOut);
	xr16_ud_t <= read_r;
	xr16_ld_t <= read_r;
	sdram_rd <= ramrd;
	sdram_wr <= ramwr;

	process(sysReset, sysClk)
	begin
		if (sysReset=YES) then
				xr16_rdy     <= NO;
	      	addr_r       <= "00000000000000000000000";  -- read at address 0
				ramrd        <= YES;
				ramwr        <= NO;
				read_r       <= YES;
				readRAMData  <= NO;
				writeRAMData <= NO;
				memoryState  <= MEM_ACTIVE;

		elsif (rising_edge(sysClk)) then
			
			case memoryState is

				when MEM_ACTIVE =>
					if (sdram_done=YES) then
						xr16_rdy    <= YES;
						ramrd       <= NO;
						ramwr       <= NO;
						memoryState <= MEM_READY;
					end if;

				when MEM_READY =>
					if(xr16_mem_ce=YES) then
						xr16_rdy     <= NO;
						addr_r       <= unsigned(xr16_addr_nxt(15 downto 1));
						ramrd        <= xr16_read_nxt;
						ramwr        <= not xr16_read_nxt;
						read_r       <= xr16_read_nxt;
						readRAMData  <= xr16_read_nxt and xr16_dbus_nxt;
						writeRAMData <= (not xr16_read_nxt) and xr16_dbus_nxt;
						memoryState  <= MEM_ACTIVE;
					end if;
			end case;

		end if;

	end process;

	--
	-- COMPONENT CREATION
	--

	-- xr16 component
	u1: xr16
	port map(
		clk      => sysClk,
		rst      => sysReset,
		rdy      => xr16_rdy,
		ud_t     => xr16_ud_t,
		ld_t     => xr16_ld_t,
		udlt_t   => HI,                  -- keep data off datapath in cpu
		int_req  => NO,
		dma_req  => NO,
		zerodma  => NO,
		insn     => xr16_insn,
		mem_ce   => xr16_mem_ce,
		word_nxt => open,                -- ignore for now, treat all reads/writes as words
		read_nxt => xr16_read_nxt,       -- specifies if cpu wants to read or write
		dbus_nxt => xr16_dbus_nxt,
		dma      => open,
		addr_nxt => xr16_addr_nxt,
		d        => xr16_d
		);

	-- SDRAM memory controller module
	u3: sdramCntl
	generic map(
    	FREQ => 25_000,                    -- 25 MHz operation
		DATA_WIDTH => 16,
		NROWS => 4096,
		NCOLS => 512,
		HADDR_WIDTH => 23,
		SADDR_WIDTH => 12
	)
	port map(
		clkin           => pin_sysClk,   -- master clock from external clock source (unbuffered)
		bufclk          => sdram_bufclk, -- buffered master clock output
		clk0            => sysClk,       -- synchronized master clock (accounts for delays to external SDRAM)
		clk2x           => open,         -- synchronized doubled master clock
		lock            => sdram_lock,   -- valid synchronized clocks indicator
		rst             => sysReset,     -- reset
		rd              => sdram_rd,     -- host-side SDRAM read control from memory tester
		wr              => sdram_wr,     -- host-side SDRAM write control from memory tester
		done            => sdram_done,   -- SDRAM memory read/write done indicator
		hAddr           => sdram_hAddr,  -- host-side address from memory tester
		hDIn            => sdram_hDIn,   -- always send data from xsoc to memory
		hDOut           => sdram_hDOut,  -- SDRAM data output to memory tester
		sdramCntl_state => open,         -- SDRAM controller state (for diagnostics)
	
		-- SDRAM side
		sclkfb => pin_sclkfb,   -- clock feedback with added external PCB delays
		sclk   => pin_sclk,     -- synchronized clock to external SDRAM
		cke    => pin_cke,      -- SDRAM clock enable
		cs_n   => pin_cs_n,     -- SDRAM chip-select
		ras_n  => pin_ras_n,    -- SDRAM RAS
		cas_n  => pin_cas_n,    -- SDRAM CAS
		we_n   => pin_we_n,     -- SDRAM write-enable
		ba     => pin_ba,       -- SDRAM bank address
		sAddr  => pin_sAddr,    -- SDRAM address
		sData  => pin_sData,    -- SDRAM databus
		dqmh   => pin_dqmh,     -- SDRAM DQMH
		dqml   => pin_dqml      -- SDRAM DQML
	);

	-- synchronous reset.  internal reset flag is set active by config. bitstream
	-- and then gets reset after DLL clocks start.
	process(sdram_bufclk)
	begin
		if (rising_edge(sdram_bufclk)) then
			if sdram_lock='0' then
				sysReset <= '1';    -- keep in reset until DLLs start up
			else
				sysReset <= '0';
			end if;
		end if;
	end process;

	ledDigit <= 0;

   --
   -- translate digit to 7 segment
   --
	pin_led <= "0111111" when ledDigit=0 else   --0
	           "0000110" when ledDigit=1 else   --1
              "1011011" when ledDigit=2 else   --2
              "1001111" when ledDigit=3 else   --3
              "1100110" when ledDigit=4 else   --4
              "1101101" when ledDigit=5 else   --5
              "1111101" when ledDigit=6 else   --6
              "0000111" when ledDigit=7 else   --7
              "1111111" when ledDigit=8 else   --8
              "1101111" when ledDigit=9 else   --9
              "1110111" when ledDigit=10 else  --A
              "1111100" when ledDigit=11 else  --b
              "0111001" when ledDigit=12 else  --C
              "1011110" when ledDigit=13 else  --d
              "1111001" when ledDigit=14 else  --E
              "1110001" when ledDigit=15 else  --F
              "1111111";                       --(error)

end arch;
